/* eslint-disable @typescript-eslint/no-var-requires, import/no-dynamic-require, global-require */
const path = require('path');
const fs = require('fs');

const CWD = process.cwd();
const isEnvDevelopment = process.env.NODE_ENV === 'development';
const appPackage = require(path.join(CWD, 'package.json'));
const webpackOverridesPath = path.join(CWD, 'webpack.overrides.js');
const devServerOverridesPath = path.join(CWD, 'dev-server.overrides.js');
const projectTsConfigPath = path.join(CWD, 'tsconfig.json');

const getAppInfo = () => {
  if (isEnvDevelopment) {
    return {
      name: appPackage.name,
      version: appPackage.version,
    };
  }

  const versionPath = path.join(CWD, 'version.json');

  return {
    name: appPackage.name,
    version: fs.existsSync(versionPath) ? require(versionPath).version : appPackage.version,
  };
};

const config = {
  ...getAppInfo(),
  tsconfig: fs.existsSync(projectTsConfigPath) ? projectTsConfigPath : null,
  cwd: CWD,
  root: path.resolve(CWD, './'),
  src: path.resolve(CWD, './src'),
  app: path.resolve(CWD, './src/index.tsx'),
  template: path.resolve(CWD, './src/index.html'),
  public: path.resolve(CWD, './public'),
  output: path.resolve(CWD, './dist'),
  callfromweb: path.resolve(CWD, '../cfw'),
  webpackOverrides: fs.existsSync(webpackOverridesPath) ? require(webpackOverridesPath) : {},
  devServerOverrides: fs.existsSync(devServerOverridesPath) ? require(devServerOverridesPath) : {},
};

module.exports = config;
/* eslint-aneble @typescript-eslint/no-var-requires, import/no-dynamic-require, global-require */
