/* eslint-env node */
/* eslint-disable import/no-extraneous-dependencies, global-require, @typescript-eslint/no-var-requires, import/no-dynamic-require */

const path = require('path');
const webpack = require('webpack');
const merge = require('lodash.merge');
const HtmlPlugin = require('html-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const {CleanWebpackPlugin} = require('clean-webpack-plugin');
const ForkTsCheckerWebpackPlugin = require('fork-ts-checker-webpack-plugin');

const config = require('./app-config');

const mode = 'production';

let webpackProd = {
  mode,
  devtool: false,
  entry: {
    app: config.app,
  },
  output: {
    path: config.output,
    publicPath: '/',
    filename: '[contenthash:8].js',
    chunkFilename: '[contenthash:8].chunk.js',
  },
  optimization: {
    minimize: true,
    minimizer: [
      new TerserPlugin({
        parallel: true,
      }),
    ],
    splitChunks: {
      chunks: 'all',
    },
  },
  module: {
    rules: [
      {
        test: /\.(js|jsx|ts|tsx)$/,
        exclude: /node_modules\/(?!@callfromwebi*)/,
        loader: 'ts-loader',
        options: {
          transpileOnly: true,
          happyPackMode: true,
          allowTsInNodeModules: true,
        },
      },
      {
        test: /\.(jpe?g|png|gif|ico|webp)$/i,
        type: 'asset/resource',
      },
      {
        test: /\.(css)$/i,
        type: 'asset/source',
      },
      {
        test: /\.svg/,
        exclude: /node_modules\/(?!@callfromwebi*)/,
        loader: 'svg-react-loader',
      },
    ],
  },
  resolve: {
    extensions: ['.js', '.jsx', '.ts', '.tsx', '.json'],
    modules: [path.resolve(config.cwd, './node_modules'), 'node_modules'],
    alias: {
      src: config.src,
    },
  },
  plugins: [
    new ForkTsCheckerWebpackPlugin(),
    new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
    new HtmlPlugin({
      template: config.template,
      base: '/',
      minify: true,
    }),
    new webpack.DefinePlugin({
      BUILDINFO: {
        name: JSON.stringify(config.name),
        version: JSON.stringify(config.version),
        timestamp: JSON.stringify(new Date().toISOString()),
      },
      'process.env.NODE_ENV': JSON.stringify(mode),
    }),
    new CleanWebpackPlugin({cleanOnceBeforeBuildPatterns: config.output}),
    new CopyPlugin({patterns: [{from: config.public, to: `${config.output}/public`}]}),
  ].filter(Boolean),
};

webpackProd = merge(webpackProd, config.webpackOverrides.production);

module.exports = webpackProd;

/* eslint-enable import/no-extraneous-dependencies, global-require, @typescript-eslint/no-var-requires, import/no-dynamic-require */
